//Int32.h: Holds the Int32 union that, when used properly, is a 40%-60% speed increase to 2 short ints
namespace Bitmath32
{
//Values of the Bits
#define _B01 1
#define _B02 2
#define _B03 4
#define _B04 8
#define _B05 16
#define _B06 32
#define _B07 64
#define _B08 128
#define _B09 256
#define _B10 512
#define _B11 1024
#define _B12 2048
#define _B13 4096
#define _B14 8192
#define _B15 16384
#define _B16 32768
#define _B17 645536
#define _B18 131072
#define _B19 262144
#define _B20 524288
#define _B21 1048576
#define _B22 2097152
#define _B23 4194304
#define _B24 8388608
#define _B25 16777216
#define _B26 33554432
#define _B27 67108864
#define _B28 134217728
#define _B29 268435456
#define _B30 536870912
#define _B31 1073741824
#define _B32 2147483648
unsigned __int32 Bits[32]={_B01,_B02,_B03,_B04,_B05,_B06,_B07,_B08,_B09,_B10,_B11,_B12,_B13,_B14,_B15,_B16,_B17,_B18,_B19,_B20,_B21,_B22,_B23,_B24,_B25,_B26,_B27,_B28,_B29,_B30,_B31,_B32};
unsigned __int8 numberbits=32;
};
union Int32     //Union for claculating two things with 16bit accuracy 
                //instead of one thing with 32bit accuracy
                //Can also be used for calculating 4 things with 8Bit accuracy
                //or for storing 32bools in one 32bit variable
{
    unsigned __int32 i32;
    unsigned __int16 i16[2];
    unsigned __int8 i8[4];
    Int32() : i32(0) {};
    Int32(unsigned __int32 i1) : i32(i1) {};
    Int32(unsigned __int16 i1, unsigned __int16 i2) {i16[0]=i1; i16[1]=i2;};
    bool GetBit(int Number) //Number begins with 1 and ends with 32
    {
        if(i32&Bitmath32::Bits[Number-1])
            return true;
        return false;
    };
    void SetBit(int Number, bool state) //Number begins with 1 and ends with 32
    {
        switch(state)
        {
            case true:  i32=i32|Bitmath32::Bits[Number-1]; break;
            case false: i32=i32&(Bitmath32::Bits[Number-1]^0xFFFFFFFF); break;
        };
    };
    //Usage:
    //c.i32=a.i32+b.i32;        is the same as c.i16[0]=a.i16[0]+b.i16[0]; c.i16[1]=a.i16[1]+b.i16[1];
    //c.i32=a.i32-b.i32;        is the same as c.i16[0]=a.i16[0]-b.i16[0]; c.i16[1]=a.i16[1]-b.i16[1];
    //c.i32=a.i32*b.i32;        is the same as c.i16[0]=a.i16[0]*b.i16[0]; c.i16[1]=a.i16[1]*b.i16[1];
    //c.i32=a.i32/b.i32;        is the same as c.i16[0]=a.i16[0]/b.i16[0]; c.i16[1]=a.i16[1]/b.i16[1];
    //a.i32|b.i32;              is the same as a.i16[0]|b.i16[0]; a.i16[1]|b.i16[1];
    //a.i32!b.i32;              is the same as a.i16[0]!b.i16[0]; a.i16[1]!b.i16[1];
    //a.i32&b.i32;              is the same as a.i16[0]&b.i16[0]; a.i16[1]&b.i16[1];
    //a.i32^b.i32;              is the same as a.i16[0]^b.i16[0]; a.i16[1]^b.i16[1];
    //(a.i32||b.i32)            is the same as (a.i16[0]||b.i16[0]||a.i16[1]||b.i16[1])
    //(a.i32&&b.i32)            is the same as (a.i16[0]&&b.i16[0]&&a.i16[1]&&b.i16[1])
    //(a.i32^^b.i32)            is the same as (a.i16[0]||b.i16[0])^^(a.i16[1]||b.i16[1])
    //(a.i32==b.i32)            is the same as ((a.i16[0]==b.i16[0])&&(a.i16[1]==b.i16[1]))
    //(a.i32!=b.i32)            is the same as ((a.i16[0]!=b.i16[0])&&(a.i16[1]!=b.i16[1]))
    //(a.i32>b.i32)             is NOT the same as ((a.i16[0]>b.i16[0])&&(a.i16[1]>b.i16[1]))
    //(a.i32<b.i32)             is NOT the same as ((a.i16[0]<b.i16[0])&&(a.i16[1]<b.i16[1]))
    //(a.i32>=b.i32)            is NOT the same as ((a.i16[0]>=b.i16[0])&&(a.i16[1]>=b.i16[1]))
    //(a.i32<=b.i32)            is NOT the same as ((a.i16[0]<=b.i16[0])&&(a.i16[1]<=b.i16[1]))
    //WARNING: DO NOT WRITE OPERATORS WICH YOU WILL USE THEN!!! THIS WILL DESTROY ANY SPEED INCREASE!!!
};
